"""
Groknroll Prompts Module

Comprehensive system prompts for agent intelligence.
"""

from groknroll.prompts.system import (
    PROMPT_ANTHROPIC,
    PROMPT_AGENTIC,
    PROMPT_PLAN,
    PROMPT_EXPLORE,
    get_provider_prompt,
    get_agent_prompt,
    build_system_prompt,
    get_tool_usage_reminder,
    get_code_quality_reminder,
)

__all__ = [
    "PROMPT_ANTHROPIC",
    "PROMPT_AGENTIC",
    "PROMPT_PLAN",
    "PROMPT_EXPLORE",
    "get_provider_prompt",
    "get_agent_prompt",
    "build_system_prompt",
    "get_tool_usage_reminder",
    "get_code_quality_reminder",
]
